# pre-industrial GHG concentrations in their default units.
# Source: http://www.pik-potsdam.de/~mmalte/rcps. Year 1765 in all cases. 
# TODO: needs to be updated for AR6/CMIP6 based on Meinshausen et al., 2017
CO2         = 278.05158  # ppm
CH4         = 721.89411  # ppb
N2O         = 272.95961  # ppb
CF4         = 35.        # ppt
C2F6        = 0.         # ppt
C3F8        = 0.         # ppt
C4F10       = 0.         # ppt
C5F12       = 0.         # ppt
C6F14       = 0.         # ppt
C7F16       = 0.         # ppt
C8F18       = 0.         # ppt
C_C4F8      = 0.         # ppt
HFC23       = 0.         # ppt
HFC32       = 0.         # ppt
HFC43_10    = 0.         # ppt
HFC43_10MEE = 0.         # ppt
HFC125      = 0.         # ppt
HFC134A     = 0.         # ppt
HFC143A     = 0.         # ppt
HFC152A     = 0.         # ppt
HFC227EA    = 0.         # ppt
HFC236FA    = 0.         # ppt
HFC245FA    = 0.         # ppt
HFC365MFC   = 0.         # ppt
SF6         = 0.         # ppt
NF3         = 0.         # ppt
SO2F2       = 0.         # ppt
CFC11       = 0.         # ppt
CFC12       = 0.         # ppt
CFC113      = 0.         # ppt
CFC114      = 0.         # ppt
CFC115      = 0.         # ppt
CARB_TET    = 0.         # ppt
CCL4        = CARB_TET   # ppt
MCF         = 0.         # ppt
CH3CCL3     = MCF        # ppt
HCFC22      = 0.         # ppt
HCFC141B    = 0.         # ppt
HCFC142B    = 0.         # ppt
HALON1211   = 0.         # ppt
HALON1202   = 0.         # ppt
HALON1301   = 0.         # ppt
HALON2402   = 0.         # ppt
CH3BR       = 5.8        # ppt
CH3CL       = 480.       # ppt
CH2CL2      = 0.         # ppt
CHCL3       = 0.         # ppt

# This is the list of gases included in the RCPs/AR5/CMIP5.
aslist   = [CO2, CH4, N2O, CF4, C2F6, C6F14, HFC23, HFC32, HFC43_10, HFC125,
            HFC134A, HFC143A, HFC227EA, HFC245FA, SF6, CFC11, CFC12, CFC113,
            CFC114, CFC115, CARB_TET, MCF, HCFC22, HCFC141B, HCFC142B,
            HALON1211, HALON1202, HALON1301, HALON2402, CH3BR, CH3CL]
