# number of chlorine atoms in each species
CFC11     = 3
CFC12     = 2
CFC113    = 3
CFC114    = 2
CFC115    = 1
CARB_TET  = 4
CCL4      = CARB_TET
MCF       = 3
CH3CCL3   = MCF
HCFC22    = 1
HCFC141B  = 2
HCFC142B  = 1
HALON1211 = 1
HALON1202 = 0
HALON1301 = 0
HALON2402 = 0
CH3BR     = 0
CH3CL     = 1
CH2CL2    = 2
CHCL3     = 3

aslist    = [CFC11, CFC12, CFC113, CFC114, CFC115, CARB_TET, MCF, HCFC22,
             HCFC141B, HCFC142B, HALON1211, HALON1202, HALON1301, HALON2402,
             CH3BR, CH3CL]

